@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: fnt.cxx,v 1.4 2001/04/09 00:15:13 deca Exp $
+     $Id: fnt.cxx,v 1.12 2002/10/26 14:24:59 sjbaker Exp $
 */
 
 #include "fntLocal.h"
@@ -79,46 +79,48 @@ float fntTexFont::low_putch ( sgVec3 curpos, float pointsize,
 
   if ( ! exists [ cc ] )
     return 0.0f ;
-
+  
   glBegin ( GL_TRIANGLE_STRIP ) ;
     glTexCoord2f ( t_left [cc], t_bot[cc] ) ;
     glVertex3f   ( curpos[0] +          v_left [cc] * pointsize,
-		   curpos[1] +          v_bot  [cc] * pointsize,
-		   curpos[2] ) ;
+                   curpos[1] +          v_bot  [cc] * pointsize,
+                   curpos[2] ) ;
 
     glTexCoord2f ( t_left [cc], t_top[cc] ) ;
-    glVertex3f   ( curpos[0] + italic + v_left [cc] * pointsize,
-		   curpos[1] +          v_top  [cc] * pointsize,
-		   curpos[2] ) ;
+    glVertex3f   ( curpos[0] + (italic + v_left [cc]) * pointsize,
+                   curpos[1] +           v_top  [cc]  * pointsize,
+                   curpos[2] ) ;
 
     glTexCoord2f ( t_right[cc], t_bot[cc] ) ;
     glVertex3f   ( curpos[0] +          v_right[cc] * pointsize,
-		   curpos[1] +          v_bot  [cc] * pointsize,
-		   curpos[2] ) ;
+                   curpos[1] +          v_bot  [cc] * pointsize,
+                   curpos[2] ) ;
 
     glTexCoord2f ( t_right[cc], t_top[cc] ) ;
-    glVertex3f   ( curpos[0] + italic + v_right[cc] * pointsize,
-		   curpos[1] +          v_top  [cc] * pointsize,
-		   curpos[2] ) ;
+    glVertex3f   ( curpos[0] + (italic + v_right[cc]) * pointsize,
+                   curpos[1] +           v_top  [cc]  * pointsize,
+                   curpos[2] ) ;
   glEnd () ;
 
-  float ww = ( gap + ( fixed_pitch ? width : v_right[cc] ) ) * pointsize ;
+  float ww = ( gap + ( fixed_pitch ? width : widths[cc] ) ) * pointsize ;
   curpos[0] += ww ;
   return ww ;
 }
 
 
 
-void fntTexFont::setGlyph ( char c,
-		float tex_left, float tex_right,
-		float tex_bot , float tex_top  ,
-		float vtx_left, float vtx_right,
-		float vtx_bot , float vtx_top  )
+void fntTexFont::setGlyph ( char c, float wid,
+        float tex_left, float tex_right,
+        float tex_bot , float tex_top  ,
+        float vtx_left, float vtx_right,
+        float vtx_bot , float vtx_top  )
 {
   unsigned int cc = (unsigned char) c ;
 
   exists[cc] = FNT_TRUE ;
 
+  widths[cc] = wid;
+
   t_left[cc] = tex_left ; t_right[cc] = tex_right ;
   t_bot [cc] = tex_bot  ; t_top  [cc] = tex_top   ;
 
@@ -127,16 +129,18 @@ void fntTexFont::setGlyph ( char c,
 }
 
 
-int fntTexFont::getGlyph ( char c,
-		float *tex_left, float *tex_right,
-		float *tex_bot , float *tex_top  ,
-		float *vtx_left, float *vtx_right,
-		float *vtx_bot , float *vtx_top  )
+int fntTexFont::getGlyph ( char c, float* wid,
+        float *tex_left, float *tex_right,
+        float *tex_bot , float *tex_top  ,
+        float *vtx_left, float *vtx_right,
+        float *vtx_bot , float *vtx_top  )
 {
   unsigned int cc = (unsigned char) c ;
 
   if ( ! exists[cc] ) return FNT_FALSE ;
 
+  if ( wid       != NULL ) *wid       = widths [cc] ;
+
   if ( tex_left  != NULL ) *tex_left  = t_left [cc] ;
   if ( tex_right != NULL ) *tex_right = t_right[cc] ;
   if ( tex_bot   != NULL ) *tex_bot   = t_bot  [cc] ;
@@ -159,15 +163,14 @@ void fntTexFont::getBBox ( const char *s,
   float h_pos = 0.0f ;
   float v_pos = 0.0f ;
   float l, r, b, t ;
-  float max_r, max_b ;
 
-  l = r = max_r = b = max_b = t = 0.0f ;
+  l = r = b = t = 0.0f ;
 
   while ( *s != '\0' )
   {
     if ( *s == '\n' )
     {
-      r = h_pos = 0.0f ;
+      h_pos = 0.0f ;
       v_pos -= 1.333f ;
       s++ ;
       continue ;
@@ -188,8 +191,6 @@ void fntTexFont::getBBox ( const char *s,
         r += 0.5f ;
         h_pos += 0.5f ;
 
-        if ( max_r < r ) max_r = r ;
-
         continue ;
       }
     }
@@ -205,16 +206,14 @@ void fntTexFont::getBBox ( const char *s,
     else
     {
       if ( l >       h_pos + v_left [cc]+italic ) l =       h_pos + v_left [cc] + italic ;
-      if ( r < gap + h_pos + v_right[cc]+italic ) r = gap + h_pos + v_right[cc]          ;
+      if ( r < gap + h_pos + v_right[cc] )        r = gap + h_pos + v_right[cc]          ;
     }
 
+    
     if ( b > v_pos + v_bot [cc] ) b = v_pos + v_bot [cc] ;
     if ( t < v_pos + v_top [cc] ) t = v_pos + v_top [cc] ;
 
-    if ( max_b > b ) max_b = b ;
-    if ( max_r < r ) max_r = r ;
-
-    h_pos += gap + ( fixed_pitch ? width : v_right[cc] ) ;
+    h_pos += gap + ( fixed_pitch ? width : widths[cc] ) ;
   }
 
   if ( left  != NULL ) *left  = l * pointsize ;
@@ -227,7 +226,7 @@ void fntTexFont::getBBox ( const char *s,
 void fntTexFont::puts ( sgVec3 curpos, float pointsize, float italic, const char *s )
 {
   SGfloat origx = curpos[0] ;
-	
+    
   if ( ! bound )
     bind_texture () ;
 
@@ -245,4 +244,9 @@ void fntTexFont::puts ( sgVec3 curpos, float pointsize, float italic, const char
   }
 }
 
+void fntInit ()
+{
+  /* Empty right now */
+}
+
 
