@@ -1,29 +1,32 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
- 
+     Copyright (C) 1998,2002  Steve Baker
+
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
      License as published by the Free Software Foundation; either
      version 2 of the License, or (at your option) any later version.
- 
+
      This library is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Library General Public License for more details.
- 
+
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- 
+
      For further information visit http://plib.sourceforge.net
 
-     $Id: puButtonBox.cxx,v 1.8 2001/06/19 02:42:18 sjbaker Exp $
+     $Id: puButtonBox.cxx,v 1.15 2002/09/02 06:05:45 sjbaker Exp $
 */
 
 
 #include "puLocal.h"
 
+UL_RTTI_DEF1(puButtonBox,puObject)
+
+
 puButtonBox::puButtonBox ( int minx, int miny, int maxx, int maxy,
                 char **labels, int one_button ) :
                     puObject ( minx, miny, maxx, maxy )
@@ -33,8 +36,21 @@ puButtonBox::puButtonBox ( int minx, int miny, int maxx, int maxy,
 
   button_labels = labels ;
 
-  for ( num_kids = 0 ; button_labels [ num_kids ] != NULL ; num_kids++ )
-    /* Count number of labels */ ;
+  newList ( labels ) ;
+}
+
+
+void puButtonBox::newList ( char ** _list )
+{
+  button_labels = _list ;
+
+  if ( button_labels == NULL )
+    num_kids = 0 ;
+  else
+    for ( num_kids = 0 ; button_labels [ num_kids ] != NULL ; num_kids++ )
+      /* Count number of items */ ;
+
+  puPostRefresh() ;
 }
 
 
@@ -51,7 +67,7 @@ int puButtonBox::checkKey ( int key, int updown )
     puDeactivateWidget () ;
   }
 
-  int v = getValue () ;
+  int v = getIntegerValue () ;
 
   if ( ! one_only )
     v = ~v ;
@@ -91,7 +107,7 @@ int puButtonBox::checkHit ( int /*button*/, int updown, int x, int y )
   if ( one_only )
     setValue ( i ) ;
   else
-    setValue ( getValue () ^ ( 1 << i ) ) ;
+    setValue ( getIntegerValue () ^ ( 1 << i ) ) ;
 
   puSetActiveWidget ( this, x, y ) ;
   invokeCallback () ;
@@ -103,7 +119,7 @@ void puButtonBox::draw ( int dx, int dy )
 {
   if ( !visible || ( window != puGetWindow () ) ) return ;
 
-  abox . draw ( dx, dy, style, colour, isReturnDefault() ) ;
+  abox.draw ( dx, dy, style, colour, isReturnDefault(), border_thickness ) ;
 
   if ( r_cb )
     r_cb ( this, dx, dy, render_data ) ;
@@ -113,16 +129,16 @@ void puButtonBox::draw ( int dx, int dy )
     {
       puBox tbox ;
 
-      tbox . min [ 0 ] = abox.min [ 0 ] + PUSTR_LGAP + PUSTR_LGAP ;
-      tbox . min [ 1 ] = abox.min [ 1 ] + ((abox.max[1]-abox.min[1]-PUSTR_TGAP-PUSTR_BGAP)/num_kids) * (num_kids-1-i) ;
-      tbox . max [ 0 ] = tbox.min [ 0 ] ;
-      tbox . max [ 1 ] = tbox.min [ 1 ] ;
+      tbox.min [ 0 ] = abox.min [ 0 ] + PUSTR_LGAP + PUSTR_LGAP ;
+      tbox.min [ 1 ] = abox.min [ 1 ] + ((abox.max[1]-abox.min[1]-PUSTR_TGAP-PUSTR_BGAP)/num_kids) * (num_kids-1-i) ;
+      tbox.max [ 0 ] = tbox.min [ 0 ] ;
+      tbox.max [ 1 ] = tbox.min [ 1 ] ;
 
-      if (( one_only && i == getValue() ) ||
-          ( !one_only && ((1<<i) & getValue() ) != 0 ) ) 
-        tbox . draw ( dx, dy + PUSTR_BGAP + PUSTR_BGAP, -PUSTYLE_RADIO, colour, FALSE ) ;
+      if (( one_only && i == getIntegerValue() ) ||
+          ( !one_only && ((1<<i) & getIntegerValue() ) != 0 ) ) 
+        tbox.draw ( dx, dy + PUSTR_BGAP + PUSTR_BGAP, -PUSTYLE_RADIO, colour, FALSE, border_thickness ) ;
       else
-        tbox . draw ( dx, dy + PUSTR_BGAP + PUSTR_BGAP, PUSTYLE_RADIO, colour, FALSE ) ;
+        tbox.draw ( dx, dy + PUSTR_BGAP + PUSTR_BGAP, PUSTYLE_RADIO, colour, FALSE, border_thickness ) ;
 
       /* If greyed out then halve the opacity when drawing the label and legend */
 
@@ -134,9 +150,9 @@ void puButtonBox::draw ( int dx, int dy )
                     colour [ PUCOL_LEGEND ][2],
                     colour [ PUCOL_LEGEND ][3] / 2.0f ) ; /* 50% more transparent */
 
-      puDrawString ( legendFont, button_labels[i],
+      legendFont.drawString ( button_labels[i],
                      dx + tbox.min[0] + PU_RADIO_BUTTON_SIZE + PUSTR_LGAP,
-                     dy + tbox.min[1] + puGetStringDescender(legendFont) + PUSTR_BGAP  + PUSTR_BGAP) ;
+                     dy + tbox.min[1] + legendFont.getStringDescender() + PUSTR_BGAP  + PUSTR_BGAP) ;
     }
   }
 
