@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,13 +13,19 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: puLocal.h,v 1.7 2001/04/12 03:36:12 deca Exp $
+     $Id: puLocal.h,v 1.14 2004/03/21 19:03:23 sjbaker Exp $
 */
 
 
+#define PU_USE_NONE 1
 #include "pu.h"
+
+
+void puSetPasteBuffer  ( const char *ch ) ;
+char *puGetPasteBuffer () ;
+
