@@ -1,32 +1,34 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
- 
+     Copyright (C) 1998,2002  Steve Baker
+
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
      License as published by the Free Software Foundation; either
      version 2 of the License, or (at your option) any later version.
- 
+
      This library is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
      Library General Public License for more details.
- 
+
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- 
+
      For further information visit http://plib.sourceforge.net
 
-     $Id: puPopupMenu.cxx,v 1.7 2001/06/19 02:42:18 sjbaker Exp $
+     $Id: puPopupMenu.cxx,v 1.18 2003/06/26 11:31:07 puggles Exp $
 */
 
+
 #include "puLocal.h"
 
-#define PUMENU_BUTTON_HEIGHT       25
-#define PUMENU_BUTTON_EXTRA_WIDTH  25
+UL_RTTI_DEF1(puPopupMenu,puPopup)
 
-puObject *puPopupMenu::add_item ( const char *str, puCallback cb )
+
+puObject *puPopupMenu::add_item ( const char *str, puCallback _cb,
+                                  void *_user_data )
 {
   int w, h ;
   getSize ( &w, &h ) ;
@@ -36,8 +38,9 @@ puObject *puPopupMenu::add_item ( const char *str, puCallback cb )
 		       colour[PUCOL_FOREGROUND][1],
 		       colour[PUCOL_FOREGROUND][2],
 		       colour[PUCOL_FOREGROUND][3] ) ;
-  b->setCallback     ( cb ) ;
-  b->setLegendPlace  ( PUPLACE_LEFT ) ;
+  b->setCallback     ( _cb ) ;
+  b->setLegendPlace  ( PUPLACE_LOWER_LEFT ) ;
+  b->setUserData     ( _user_data ) ;
   recalc_bbox () ;
   return b ;
 }
@@ -61,14 +64,14 @@ void puPopupMenu::close ( void )
 
   int w, h ;
 
-  for ( ob = dlist ; ob != NULL ; ob = ob -> next )
+  for ( ob = dlist ; ob != NULL ; ob = ob -> getNextObject() )
   {
     ob -> getSize ( &w, &h ) ;
 
     if ( w > widest ) widest = w ;
   }
 
-  for ( ob = dlist ; ob != NULL ; ob = ob -> next )
+  for ( ob = dlist ; ob != NULL ; ob = ob -> getNextObject() )
   {
     ob -> getSize ( &w, &h ) ;
     ob -> setSize ( widest, h ) ;
@@ -95,7 +98,7 @@ int puPopupMenu::checkKey ( int key, int updown )
 
     /* Turn everything off ready for next time. */
 
-    for ( puObject *bo = dlist ; bo != NULL ; bo = bo->next )
+    for ( puObject *bo = dlist ; bo != NULL ; bo = bo -> getNextObject() )
       bo -> clrValue () ;
   }
 
@@ -106,10 +109,12 @@ int puPopupMenu::checkKey ( int key, int updown )
     the click order is the same as the DRAW order.
   */
 
-  for ( bo = dlist ; bo->next != NULL ; bo = bo->next )
+  for ( bo = dlist ;
+        bo -> getNextObject() != NULL ;
+        bo = bo -> getNextObject() )
     /* Find the last object in our list. */ ;
 
-  for ( ; bo != NULL ; bo = bo->prev )
+  for ( ; bo != NULL ; bo = bo -> getPrevObject() )
     if ( bo -> checkKey ( key, updown ) )
       return TRUE ;
 
@@ -122,9 +127,16 @@ int puPopupMenu::checkHit ( int button, int updown, int x, int y )
   if ( dlist == NULL || ! isVisible () || ! isActive () || ( window != puGetWindow () ) )
     return FALSE ;
 
-  /* Must test 'isHit' before making the menu invisible! */
+  /* If an item is going to fall out the bottom of the window and puGroup will  */
+  /* move it upward (around line 303 of puGroup.cxx), then make sure checkhit   */
+  /* realizes the thing has moved -- JCJ and Fay 5 June 2002                    */
 
-  int hit = isHit ( x, y ) ;
+  int absx, absy ;
+  getAbsolutePosition (&absx, &absy) ;
+  if ( absy < 0 )
+    y += absy ;
+
+  /* Must test 'isHit' before making the menu invisible! */
 
   /*
    * June 17th, 1998, Shammi :
@@ -138,6 +150,12 @@ int puPopupMenu::checkHit ( int button, int updown, int x, int y )
     changing sizes.
   */
 
+  int hit = isHit ( x, y ) ;
+
+  /* If the mouse is clicked on another widget, hide the window - JCJ 18 Jun 2002 */
+  if ( updown != PU_DRAG && !hit )
+    hide () ;
+
   recalc_bbox();
   x -= abox.min[0] ;
   y -= abox.min[1] ;
@@ -165,7 +183,7 @@ int puPopupMenu::checkHit ( int button, int updown, int x, int y )
      * out of the menu.
      */
 
-    for ( puObject *bo = dlist ; bo != NULL ; bo = bo->next )
+    for ( puObject *bo = dlist ; bo != NULL ; bo = bo -> getNextObject() )
     {
       if ( ! hit )
         bo -> checkHit ( button, updown, x , y ) ;
@@ -178,7 +196,7 @@ int puPopupMenu::checkHit ( int button, int updown, int x, int y )
     return FALSE ;
 
   puObject *bo ;
-  
+  int objecthit = FALSE ;
   /*
     We have to walk the list backwards to ensure that
     the click order is the same as the DRAW order.
@@ -189,14 +207,34 @@ int puPopupMenu::checkHit ( int button, int updown, int x, int y )
    * clear it
    */
 
-  for ( bo = dlist ; bo->next != NULL ; bo = bo->next )
+  for ( bo = dlist ;
+        bo -> getNextObject() != NULL ;
+        bo = bo -> getNextObject() )
     if ( updown == PU_DRAG && ! bo -> checkHit ( button, updown, x, y ) )
       bo -> clrValue () ;
 
     /* Find the last object in our list. */ ;
 
-  for ( ; bo != NULL ; bo = bo->prev )
+    /* Following code permits highlighting and lowlighting */
+    /* of oneshots underneath a PopupMenu.                 */
+    /*                           - JCJ and Fay 31 May 2002 */
+
+  for ( ; bo != NULL ; bo = bo -> getPrevObject() ) 
+  {
+    puBox *box = bo->getABox () ;
+    if ( ( x > box->min[0] ) && ( x < box->max[0] ) &&
+         ( y > box->min[1] ) && ( y < box->max[1] ) )
+      bo->highlight () ;
+    else
+      bo->lowlight () ;
+  /* Since actually RETURNing here would break the for loop early and kill the */
+  /* correct lowlighting, just set a var to check later. - JCJ 3 Jun 2002      */
     if ( bo -> checkHit ( button, updown, x, y ) )
+      objecthit = TRUE ;
+  }
+
+  /* If the object was hit in the above for loop, then it.. was hit.*/
+  if (objecthit)
       return TRUE ;
 
   return FALSE ;
