@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgLoadAC.cxx,v 1.19 2001/06/13 17:47:17 sjbaker Exp $
+     $Id: ssgLoadAC.cxx,v 1.32 2003/07/11 11:16:43 puggles Exp $
 */
 
 
@@ -52,46 +52,46 @@ static sgMat4 current_matrix ;
 static sgVec2 texrep ;
 static sgVec2 texoff ;
 
-int do_material ( char *s ) ;
-int do_object   ( char *s ) ;
-int do_name     ( char *s ) ;
-int do_data     ( char *s ) ;
-int do_texture  ( char *s ) ;
-int do_texrep   ( char *s ) ;
-int do_texoff   ( char *s ) ;
-int do_rot      ( char *s ) ;
-int do_loc      ( char *s ) ;
-int do_url      ( char *s ) ;
-int do_numvert  ( char *s ) ;
-int do_numsurf  ( char *s ) ;
-int do_surf     ( char *s ) ;
-int do_mat      ( char *s ) ;
-int do_refs     ( char *s ) ;
-int do_kids     ( char *s ) ;
+static int do_material ( char *s ) ;
+static int do_object   ( char *s ) ;
+static int do_name     ( char *s ) ;
+static int do_data     ( char *s ) ;
+static int do_texture  ( char *s ) ;
+static int do_texrep   ( char *s ) ;
+static int do_texoff   ( char *s ) ;
+static int do_rot      ( char *s ) ;
+static int do_loc      ( char *s ) ;
+static int do_url      ( char *s ) ;
+static int do_numvert  ( char *s ) ;
+static int do_numsurf  ( char *s ) ;
+static int do_surf     ( char *s ) ;
+static int do_mat      ( char *s ) ;
+static int do_refs     ( char *s ) ;
+static int do_kids     ( char *s ) ;
 
-int do_obj_world ( char *s ) ;
-int do_obj_poly  ( char *s ) ;
-int do_obj_group ( char *s ) ;
-int do_obj_light ( char *s ) ;
+/*static int do_obj_world ( char *s ) ;
+static int do_obj_poly  ( char *s ) ;
+static int do_obj_group ( char *s ) ;
+static int do_obj_light ( char *s ) ;*/
 
 #define PARSE_CONT   0
 #define PARSE_POP    1
 
 struct Tag
 {
-  char *token ;
+  const char *token ;
   int (*func) ( char *s ) ;
 } ;
 
  
-void skip_spaces ( char **s )
+static void skip_spaces ( char **s )
 {
   while ( **s == ' ' || **s == '\t' )
     (*s)++ ;
 }
 
 
-void skip_quotes ( char **s )
+static void skip_quotes ( char **s )
 {
   skip_spaces ( s ) ;
 
@@ -115,7 +115,7 @@ void skip_quotes ( char **s )
 
 
 
-int search ( Tag *tags, char *s )
+static int search ( Tag *tags, char *s )
 {
   skip_spaces ( & s ) ;
 
@@ -134,14 +134,15 @@ int search ( Tag *tags, char *s )
   return 0 ;  /* Should never get here */
 }
 
-Tag top_tags [] =
+static Tag top_tags [] =
 {
   { "MATERIAL", do_material },
   { "OBJECT"  , do_object   },
+  { NULL, NULL }
 } ;
 
 
-Tag object_tags [] =
+static Tag object_tags [] =
 {
   { "name"    , do_name     },
   { "data"    , do_data     },
@@ -157,27 +158,27 @@ Tag object_tags [] =
   { NULL, NULL }
 } ;
 
-Tag surf_tag [] =
+static Tag surf_tag [] =
 {
   { "SURF"    , do_surf     },
   { NULL, NULL }
 } ;
 
-Tag surface_tags [] =
+static Tag surface_tags [] =
 {
   { "mat"     , do_mat      },
   { "refs"    , do_refs     },
   { NULL, NULL }
 } ;
 
-Tag obj_type_tags [] =
+/*static Tag obj_type_tags [] =
 {
   { "world", do_obj_world },
   { "poly" , do_obj_poly  },
   { "group", do_obj_group },
   { "light", do_obj_light },
   { NULL, NULL }
-} ;
+} ;*/
 
 
 #define OBJ_WORLD  0
@@ -185,13 +186,13 @@ Tag obj_type_tags [] =
 #define OBJ_GROUP  2
 #define OBJ_LIGHT  3
 
-int do_obj_world ( char * ) { return OBJ_WORLD ; } 
-int do_obj_poly  ( char * ) { return OBJ_POLY  ; }
-int do_obj_group ( char * ) { return OBJ_GROUP ; }
-int do_obj_light ( char * ) { return OBJ_LIGHT ; }
+/*static int do_obj_world ( char * ) { return OBJ_WORLD ; } 
+static int do_obj_poly  ( char * ) { return OBJ_POLY  ; }
+static int do_obj_group ( char * ) { return OBJ_GROUP ; }
+static int do_obj_light ( char * ) { return OBJ_LIGHT ; }*/
 
-int last_num_kids    = -1 ;
-int current_flags    = -1 ;
+static int last_num_kids    = -1 ;
+static int current_flags    = -1 ;
 
 static ssgState *get_state ( _ssgMaterial *mat )
 {
@@ -213,7 +214,21 @@ static ssgState *get_state ( _ssgMaterial *mat )
   st -> enable  ( GL_LIGHTING ) ;
   st -> setShadeModel ( GL_SMOOTH ) ;
 
-  if ( mat -> rgb[3] < 0.99 )
+  bool has_alpha = false ;
+
+  if ( current_tfname != NULL )
+  {
+    ssgTexture *tex = current_options -> createTexture ( current_tfname ) ;
+    has_alpha = tex -> hasAlpha () ;
+    st -> setTexture( tex ) ;
+    st -> enable( GL_TEXTURE_2D ) ;
+  }
+  else
+  {
+    st -> disable( GL_TEXTURE_2D ) ;
+  }
+
+  if ( mat -> rgb[3] < 0.99 || has_alpha )
   {
     st -> disable ( GL_ALPHA_TEST ) ;
     st -> enable  ( GL_BLEND ) ;
@@ -221,22 +236,16 @@ static ssgState *get_state ( _ssgMaterial *mat )
   }
   else
   {
+    st -> disable ( GL_ALPHA_TEST ) ;
     st -> disable ( GL_BLEND ) ;
     st -> setOpaque () ;
   }
 
-  if (current_tfname != NULL) {
-    st -> setTexture( current_options -> createTexture(current_tfname) ) ;
-    st -> enable( GL_TEXTURE_2D ) ;
-  } else {
-    st -> disable( GL_TEXTURE_2D ) ;
-  }
-
   return st ;
 }
 
 
-int do_material ( char *s )
+static int do_material ( char *s )
 {
   char name [ 1024 ] ;
   sgVec4 rgb  ;
@@ -285,13 +294,13 @@ int do_material ( char *s )
 }
 
 
-int do_object   ( char *  /* s */ )
+static int do_object   ( char *  /* s */ )
 {
 /*
   int obj_type = search ( obj_type_tags, s ) ;  
 */
 
-  delete current_tfname ;
+  delete [] current_tfname ;
   current_tfname = NULL ;
 
   char buffer [ 1024 ] ;
@@ -327,7 +336,7 @@ int do_object   ( char *  /* s */ )
 }
 
 
-int do_name ( char *s )
+static int do_name ( char *s )
 {
   skip_quotes ( &s ) ;
 
@@ -337,18 +346,23 @@ int do_name ( char *s )
 }
 
 
-int do_data     ( char *s )
+static int do_data ( char *s )
 {
   int len = strtol ( s, NULL, 0 ) ;
 
   current_data = new char [ len + 1 ] ;
 
   for ( int i = 0 ; i < len ; i++ )
-    current_data [ i ] = fgetc ( loader_fd ) ;
+    current_data [ i ] = getc ( loader_fd ) ;
 
   current_data [ len ] = '\0' ;
 
-  fgetc ( loader_fd ) ;  /* Final RETURN */
+  int c;
+  while ( ( c = getc( loader_fd ) ) != EOF )  /* Final RETURN */
+    if ( c != '\r' && c != '\n' ) {
+      ungetc ( c, loader_fd ) ;
+      break ;
+    }
 
   ssgBranch *br = current_options -> createBranch ( current_data ) ;
 
@@ -365,25 +379,22 @@ int do_data     ( char *s )
 }
 
 
-int do_texture  ( char *s )
+static int do_texture  ( char *s )
 {
   skip_quotes ( &s ) ;
 
-  delete current_tfname ;
+  delete [] current_tfname ;
 
   if ( s == NULL || s[0] == '\0' )
     current_tfname = NULL ;
   else
-  {
-    current_tfname = new char [ strlen(s)+1 ] ;
-    strcpy ( current_tfname, s ) ;
-  }
+    current_tfname = ulStrDup ( s ) ;
 
   return PARSE_CONT ;
 }
 
 
-int do_texrep ( char *s )
+static int do_texrep ( char *s )
 {
   if ( sscanf ( s, "%f %f", & texrep [ 0 ], & texrep [ 1 ] ) != 2 )
     ulSetError ( UL_WARNING, "ac_to_gl: Illegal texrep record." ) ;
@@ -392,7 +403,7 @@ int do_texrep ( char *s )
 }
 
 
-int do_texoff ( char *s )
+static int do_texoff ( char *s )
 {
   if ( sscanf ( s, "%f %f", & texoff [ 0 ], & texoff [ 1 ] ) != 2 )
     ulSetError ( UL_WARNING, "ac_to_gl: Illegal texoff record." ) ;
@@ -400,7 +411,7 @@ int do_texoff ( char *s )
   return PARSE_CONT ;
 }
 
-int do_rot ( char *s )
+static int do_rot ( char *s )
 {
   current_matrix [ 0 ][ 3 ] = current_matrix [ 1 ][ 3 ] = current_matrix [ 2 ][ 3 ] =
     current_matrix [ 3 ][ 0 ] = current_matrix [ 3 ][ 1 ] = current_matrix [ 3 ][ 2 ] = 0.0f ;
@@ -416,7 +427,7 @@ int do_rot ( char *s )
   return PARSE_CONT ;
 }
 
-int do_loc      ( char *s )
+static int do_loc      ( char *s )
 {
   if ( sscanf ( s, "%f %f %f", & current_matrix [ 3 ][ 0 ], & current_matrix [ 3 ][ 2 ], & current_matrix [ 3 ][ 1 ] ) != 3 )
     ulSetError ( UL_WARNING, "ac_to_gl: Illegal loc record." ) ;
@@ -428,18 +439,18 @@ int do_loc      ( char *s )
   return PARSE_CONT ;
 }
 
-int do_url      ( char *s )
+static int do_url      ( char *s )
 {
   skip_quotes ( & s ) ;
 
 #ifdef PRINT_URLS
-  printf ( "/* URL: \"%s\" */\n", s ) ;
+  ulSetError ( UL_DEBUG, "/* URL: \"%s\" */\n", s ) ;
 #endif
 
   return PARSE_CONT ;
 }
 
-int do_numvert  ( char *s )
+static int do_numvert  ( char *s )
 {
   char buffer [ 1024 ] ;
 
@@ -467,7 +478,7 @@ int do_numvert  ( char *s )
   return PARSE_CONT ;
 }
 
-int do_numsurf  ( char *s )
+static int do_numsurf  ( char *s )
 {
   int ns = strtol ( s, NULL, 0 ) ;
 
@@ -482,7 +493,7 @@ int do_numsurf  ( char *s )
   return PARSE_CONT ;
 }
 
-int do_surf     ( char *s )
+static int do_surf     ( char *s )
 {
   current_flags = strtol ( s, NULL, 0 ) ;
 
@@ -496,7 +507,7 @@ int do_surf     ( char *s )
 }
 
 
-int do_mat ( char *s )
+static int do_mat ( char *s )
 {
   int mat = strtol ( s, NULL, 0 ) ;
 
@@ -507,7 +518,7 @@ int do_mat ( char *s )
 }
 
 
-int do_refs     ( char *s )
+static int do_refs     ( char *s )
 {
   int nrefs = strtol ( s, NULL, 0 ) ;
   char buffer [ 1024 ] ;
@@ -583,7 +594,7 @@ int do_refs     ( char *s )
   return PARSE_POP ;
 }
 
-int do_kids ( char *s )
+static int do_kids ( char *s )
 {
   last_num_kids = strtol ( s, NULL, 0 ) ;
 
@@ -671,10 +682,11 @@ ssgEntity *ssgLoadAC ( const char *fname, const ssgLoaderOptions* options )
       search ( top_tags, s ) ;
   }
 
-  delete current_tfname ;
+  delete [] current_tfname ;
   current_tfname = NULL ;
   delete [] vtab ;
   fclose ( loader_fd ) ;
 
   return current_branch ;
 }
+
