@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgLoadX.cxx,v 1.14 2001/04/12 03:38:15 deca Exp $
+     $Id: ssgLoadX.cxx,v 1.21 2003/07/26 09:33:18 wolfram_kuss Exp $
 */
 
 //
@@ -36,13 +36,13 @@
 typedef int HandlerFunctionType(const char *sName, const char *firstToken);
 
 
-char *globEmpty="";
+static char *globEmpty="";
 
 static ssgBranch *curr_branch_;
 
 struct EntityType
 {
-  char * sName;
+  const char * sName;
 	HandlerFunctionType *HandleEntity;
 	int bMayBeIgnored;
 } ;
@@ -105,7 +105,7 @@ static int Ascii2Float(SGfloat &retVal, const char *token, const char* name )
 	}
 }
 
-int HandleHeader(const char * /* sName */, const char *firstToken)
+static int HandleHeader(const char * /* sName */, const char *firstToken)
 {
 	//parser.expectNextToken("{");
 	int Dummy;
@@ -123,7 +123,7 @@ int HandleHeader(const char * /* sName */, const char *firstToken)
 	return TRUE;
 }
 
-void IgnoreEntity(int startLevel)
+static void IgnoreEntity(int startLevel)
 // startLevel should be 0 when you are "in front of the {" (normally)
 // or 1 when you have already parsed the "{"
 { 
@@ -149,14 +149,14 @@ void IgnoreEntity(int startLevel)
 	}      
 }
 
-int HandleMesh(const char *sName, const char *firstToken);
-int HandleMeshMaterialList(const char *sName, const char *firstToken);
-int HandleTextureCoords(const char *sName, const char *firstToken);
-int HandleMaterial(const char *sName, const char *firstToken);
-int HandleTextureFileName(const char *sName, const char *firstToken);
+static int HandleMesh(const char *sName, const char *firstToken);
+static int HandleMeshMaterialList(const char *sName, const char *firstToken);
+static int HandleTextureCoords(const char *sName, const char *firstToken);
+static int HandleMaterial(const char *sName, const char *firstToken);
+static int HandleTextureFileName(const char *sName, const char *firstToken);
 
 
-EntityType aEntities[] =
+static EntityType aEntities[] =
 {
 	{ "Header", HandleHeader, FALSE}, 
 	{ "Vector", NULL, FALSE}, 
@@ -190,7 +190,7 @@ EntityType aEntities[] =
 };
 
 
-int ParseEntity(char *token)
+static int ParseEntity(char *token)
 // called recursively
 { int i=0;
 
@@ -203,17 +203,19 @@ int ParseEntity(char *token)
 				{ parser.error("unexpected end fo file\n");
 					return FALSE;
 				}
-        
-				if (0!=strcmp(sNextToken, "{"))
-				{ sName=sNextToken;
-					sNextToken=parser.getNextToken(0);
-					if (0!=strcmp(sNextToken, "{"))
+        sName = NULL;
+				if (0 != strcmp(sNextToken, "{"))
+				{ sName = new char[ strlen(sNextToken) + 1 ];
+					assert ( sName );
+					strcpy(sName, sNextToken);
+					sNextToken = parser.getNextToken(0);
+					if (0 != strcmp(sNextToken, "{"))
 						parser.error("\"{\" expected\n");
 				}
-				sNextToken=parser.getNextToken(0);
+				sNextToken = parser.getNextToken(0);
 				
-				if(sNextToken[0]=='<') // UUID
-					sNextToken=parser.getNextToken(0);
+				if(sNextToken[0] == '<') // UUID
+					sNextToken = parser.getNextToken(0);
 				if ( parser.eof ) 
 				{ parser.error("unexpected end fo file\n");
 					return FALSE;
@@ -221,6 +223,8 @@ int ParseEntity(char *token)
         
 				if (!aEntities[i].HandleEntity(sName, sNextToken))
 					return FALSE;
+				if ( sName )
+					delete [] sName;
 			}
 			else
 				if (aEntities[i].bMayBeIgnored)
@@ -247,12 +251,12 @@ int ParseEntity(char *token)
 
 #define MAX_NO_VERTICES_PER_FACE 1000
 
-class ssgLoaderWriterMesh currentMesh;
+static class ssgLoaderWriterMesh currentMesh;
 
-ssgSimpleState *currentState;
+static ssgSimpleState *currentState;
 extern sgVec4 currentDiffuse;
 
-int HandleTextureFileName(const char *sName, const char *firstToken)
+static int HandleTextureFileName(const char * /*sName*/, const char *firstToken)
 {/*
 	  TextureFilename {
     "../image/box_top.gif";
@@ -277,7 +281,7 @@ int HandleTextureFileName(const char *sName, const char *firstToken)
 	return TRUE;
 }
 
-int HandleMaterial(const char *sName, const char *firstToken)
+static int HandleMaterial(const char * /*sName*/, const char *firstToken)
 // return TRUE on success
 { SGfloat power;
   int bFoundTextureFileName = FALSE;
@@ -371,7 +375,7 @@ int HandleMaterial(const char *sName, const char *firstToken)
 	return TRUE; //lint !e527
 }
 
-int HandleTextureCoords(const char * /* sName */, const char *firstToken)
+static int HandleTextureCoords(const char * /* sName */, const char *firstToken)
 {
 	u32 nNoOfVertices, i;
 
@@ -387,7 +391,7 @@ int HandleTextureCoords(const char * /* sName */, const char *firstToken)
 	  IgnoreEntity ( 1 ); // ignores TC.
 		return FALSE;
 	}
-	currentMesh.ThereAreNTCPV( nNoOfVertices ) ;
+	currentMesh.createPerVertexTextureCoordinates2( nNoOfVertices ) ;
 
 	parser.expectNextToken(";");
 	for(i=0;i<nNoOfVertices;i++)
@@ -405,13 +409,13 @@ int HandleTextureCoords(const char * /* sName */, const char *firstToken)
 		else
 			parser.expectNextToken(",");
 
-		currentMesh.addTCPV ( tv ) ;
+		currentMesh.addPerVertexTextureCoordinate2( tv ) ;
 	}
 	parser.expectNextToken("}");
 	return TRUE;
 }
 
-int HandleMeshMaterialList(const char * /* sName */, const char *firstToken)
+static int HandleMeshMaterialList(const char * /* sName */, const char *firstToken)
 {
 	u32 i, nFaceIndexes, nMaterialsRead = 0, nMaterials;
 	  
@@ -419,10 +423,10 @@ int HandleMeshMaterialList(const char * /* sName */, const char *firstToken)
 		return FALSE;
 
 	parser.expectNextToken(";");
-	currentMesh.ThereAreNMaterials( nMaterials );
+	currentMesh.createMaterials( nMaterials );
 	if (!parser.getNextUInt(nFaceIndexes, "number of Face Indexes"))
 		return FALSE;
-	currentMesh.ThereAreNMaterialIndexes( nFaceIndexes ) ;
+	currentMesh.createMaterialIndices( nFaceIndexes ) ;
 	parser.expectNextToken(";");
 
 
@@ -481,11 +485,10 @@ int HandleMeshMaterialList(const char * /* sName */, const char *firstToken)
 }
 
 
-int HandleMesh(const char * /* sName */, const char *firstToken)
+static int HandleMesh(const char * sName, const char *firstToken)
 { u32 i, j, nNoOfVertices, nNoOfVerticesForThisFace, nNoOfFaces;
 	int iVertex, aiVertices[MAX_NO_VERTICES_PER_FACE];
 	
-	
 	//char *sMeshName = parser.getNextToken("Mesh name");
 	//parser.expectNextToken("{");
 	if (! Ascii2UInt(nNoOfVertices, firstToken, "nNoOfVertices"))
@@ -493,10 +496,10 @@ int HandleMesh(const char * /* sName */, const char *firstToken)
 
 	//parser.getNextInt("number of vertices");
 
-	currentMesh.ReInit ();
-	currentMesh.ThereAreNVertices( nNoOfVertices );
-  
-	
+	currentMesh.reInit ();
+	currentMesh.setName( sName );
+	currentMesh.createVertices( nNoOfVertices );
+ 
 	parser.expectNextToken(";");
 	for(i=0;i<nNoOfVertices;i++)
 	{ 
@@ -520,7 +523,7 @@ int HandleMesh(const char * /* sName */, const char *firstToken)
 	}
 	if (!parser.getNextUInt(nNoOfFaces, "number of faces"))
 		 return FALSE;
-	currentMesh.ThereAreNFaces ( nNoOfFaces );
+	currentMesh.createFaces( nNoOfFaces );
 
   
   
@@ -553,7 +556,7 @@ int HandleMesh(const char * /* sName */, const char *firstToken)
 #ifdef NOT_PLIB
 		CreateFaceInMdi_Edit(vl, nNoOfVerticesForThisFace, aiVertices); // This line is for the "Mdi" 3D Editor, NOT for plib
 #else
-		currentMesh.AddFaceFromCArray(nNoOfVerticesForThisFace, aiVertices); 
+		currentMesh.addFaceFromIntegerArray(nNoOfVerticesForThisFace, aiVertices); 
 #endif
 	}
 	while(TRUE)
@@ -571,9 +574,40 @@ int HandleMesh(const char * /* sName */, const char *firstToken)
 
 //
 	if ( currentState == NULL )
-		currentState = new ssgSimpleState();
+	{	currentState = new ssgSimpleState();
+	  currentState->setOpaque();
+		currentState->disable(GL_BLEND);
+		currentState->disable(GL_ALPHA_TEST);
+		currentState->disable(GL_TEXTURE_2D);
+		currentState->enable(GL_COLOR_MATERIAL);
+		currentState->enable(GL_LIGHTING);
+		currentState->setShadeModel(GL_SMOOTH);
+		currentState->setMaterial(GL_AMBIENT , 0.7f, 0.7f, 0.7f, 1.0f);
+		currentState->setMaterial(GL_DIFFUSE , 0.7f, 0.7f, 0.7f, 1.0f);
+		currentState->setMaterial(GL_SPECULAR, 1.0f, 1.0f, 1.0f, 1.0f);
+		currentState->setMaterial(GL_EMISSION, 0.0f, 0.0f, 0.0f, 1.0f);
+		currentState->setShininess(50);
+/*
+		currentState -> setMaterial ( GL_AMBIENT, 0.5, 0.5, 0.5);
+   	currentState -> setMaterial ( GL_DIFFUSE,  0.7, 0.7, 0.7); // light grey
+		currentState -> setMaterial ( GL_SPECULAR, 1.0, 1.0, 1.0);
+		currentState -> setMaterial ( GL_EMISSION, 0.0, 0.0, 0.0);
+		
+		currentState -> setShininess ( 3 ) ; 
 
-	currentMesh.add2SSG(
+		//currentState -> enable ( GL_COLOR_MATERIAL ) ;
+		//currentState -> setColourMaterial ( GL_AMBIENT_AND_DIFFUSE ) ;
+
+		currentState -> enable  ( GL_LIGHTING       ) ;
+		currentState -> setShadeModel ( GL_SMOOTH ) ;
+
+		currentState -> disable ( GL_BLEND ) ;
+		currentState -> setOpaque () ;
+		currentState -> disable( GL_TEXTURE_2D );
+		*/
+	}
+
+	currentMesh.addToSSG(
 		currentState // Pfusch, kludge. NIV135
 		,
 		current_options,
@@ -581,7 +615,7 @@ int HandleMesh(const char * /* sName */, const char *firstToken)
 	return TRUE;
 }
 
-static int TwoCharsToInt(char char1, char char2)
+inline int TwoCharsToInt(char char1, char char2)
 {
   return ((int)(char1-'0'))*256+char2-'0';
 }
@@ -633,7 +667,8 @@ static int parse()
   //int startLevel = parser.level;
 	token = parser.getNextToken(0);
   while (! parser.eof )
-	{ if (firsttime)
+	{ 
+		if (firsttime)
 		{ 
 			if(!HeaderIsValid(token))
 					return FALSE;
@@ -654,6 +689,7 @@ ssgEntity *ssgLoadX ( const char *fname, const ssgLoaderOptions* options )
   ssgSetCurrentOptions ( (ssgLoaderOptions*)options ) ;
   current_options = ssgGetCurrentOptions () ;
 
+	currentState = NULL; 
   top_branch = new ssgBranch ;
 	curr_branch_ = top_branch;
 	if ( !parser.openFile( fname, &parser_spec ))
