@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,20 +13,22 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgOptimiser.cxx,v 1.16 2001/04/12 03:38:15 deca Exp $
+     $Id: ssgOptimiser.cxx,v 1.34 2004/01/20 22:21:02 smokydiamond Exp $
 */
 
 
 #include "ssgLocal.h"
 
+#include "ssgVertSplitter.h"
+
 static float optimise_vtol [3] =
 {
-  0.01f,   /* DISTANCE_SLOP = One centimeter */
+  0.001f,  /* DISTANCE_SLOP = One millimeter */
   0.04f,   /* COLOUR_SLOP = Four percent */
   0.004f,  /* TEXCOORD_SLOP = One texel on a 256 map */
 } ;
@@ -66,17 +68,26 @@ struct OptVertex
     counter = 1 ;
   }
   
+    OptVertex ( OptVertex* cloneFrom )
+    {
+        sgCopyVec3 ( vertex  , cloneFrom -> vertex ) ;
+        sgCopyVec2 ( texcoord, cloneFrom -> texcoord ) ;
+        sgCopyVec4 ( colour  , cloneFrom -> colour ) ;
+        sgSetVec3  ( normal  , 0.0f, 0.0f, 0.0f ) ;
+        counter = 1 ;
+    }
+
   int equal ( sgVec3 v, sgVec2 t, sgVec4 c, int tex_frac )
   {
-    if ( ! sgCompareVec3 ( vertex  , v, DISTANCE_SLOP ) == 0 ||
-      ! sgCompareVec4 ( colour  , c, COLOUR_SLOP   ) == 0 )
+    if ( ! sgCompareVec3 ( vertex  , v, DISTANCE_SLOP ) ||
+         ! sgCompareVec4 ( colour  , c, COLOUR_SLOP   ) )
       return FALSE ;
     
     if ( ! tex_frac )
-      return sgCompareVec2 ( texcoord, t, TEXCOORD_SLOP ) == 0 ;
+      return sgCompareVec2 ( texcoord, t, TEXCOORD_SLOP ) ;
     
-    return fabs ( frac ( texcoord[0] ) - frac ( t[0] ) ) <= TEXCOORD_SLOP &&
-      fabs ( frac ( texcoord[1] ) - frac ( t[1] ) ) <= TEXCOORD_SLOP ;
+    return ( fabs ( frac ( texcoord[0] ) - frac ( t[0] ) ) <= TEXCOORD_SLOP &&
+	     fabs ( frac ( texcoord[1] ) - frac ( t[1] ) ) <= TEXCOORD_SLOP ) ;
   }
   
   void bump () { counter++ ; }
@@ -117,8 +128,8 @@ public:
     for ( int i = 0 ; i < vnum ; i++ )
       delete vlist [ i ] ;
     
-    delete vlist ;
-    delete tlist ;
+    delete [] vlist ;
+    delete [] tlist ;
 
     if (state != NULL) ssgDeRefDelete(state); //~T.G.
   }
@@ -207,7 +218,7 @@ short OptVertexList::add ( sgVec3 v1, sgVec2 t1, sgVec4 c1,
 {
 	// If possible, this routine moves the texturecoordinates of
 	// all three vertices of a Tria so that one needs less vertices.
-	// This doesnt affect looks, but enhances the speed a bit.
+        // This doesn't affect looks, but enhances the speed a bit.
 	// This is not possible, if warpu or wrapv is FALSE
 
 	int bWrapsInBothDirections = FALSE;
@@ -292,32 +303,61 @@ short OptVertexList::add ( short v1, short v2, short v3 )
   return tnum++ ;
 }
 
-
-void OptVertexList::makeNormals()
+//
+// Splits vertices across "sharp" edges to improve the normal
+// direction.
+//
+void OptVertexList::makeNormals ()
 {
-  short i ;
-  
-  for ( i = 0 ; i < vnum ; i++ )
-    sgSetVec3 ( vlist [ i ] -> normal, 0.0f, 0.0f, 0.0f ) ;
-  
-  for ( i = 0 ; i < tnum ; i++ )
-  {
-    sgVec3 tmp ;
+    int i, j ;
+    ssgVertSplitter vs ( vnum, tnum ) ;
     
-    sgMakeNormal ( tmp, vlist [ tlist [ i*3+ 0 ] ] -> vertex,
-      vlist [ tlist [ i*3+ 1 ] ] -> vertex,
-      vlist [ tlist [ i*3+ 2 ] ] -> vertex ) ;
-    
-    sgAddVec3 ( vlist [ tlist [ i*3+ 0 ] ] -> normal, tmp ) ;
-    sgAddVec3 ( vlist [ tlist [ i*3+ 1 ] ] -> normal, tmp ) ;
-    sgAddVec3 ( vlist [ tlist [ i*3+ 2 ] ] -> normal, tmp ) ;
-  }
-  
-  for ( i = 0 ; i < vnum ; i++ )
-    if ( sgScalarProductVec2 ( vlist[i]->normal, vlist[i]->normal ) < 0.001 )
-      sgSetVec3 ( vlist[i]->normal, 0.0f, 0.0f, 1.0f ) ;
-    else
-      sgNormaliseVec3 ( vlist [ i ] -> normal ) ;
+    // Copy in the vertex and triangle data
+    for ( i = 0 ; i < vnum ; i ++ )
+        sgCopyVec3 ( vs.vert ( i ), vlist[ i ] -> vertex ) ;
+
+    for ( i = 0 ; i < tnum ; i ++ ) {
+        short* t = tlist + 3 * i ;
+        vs.setTri ( i , t[0], t[1], t[2] );
+    }
+
+    // Run the algorithm.  Get the "sharp" angle from somewhere
+    // intelligent, instead of hardcoding it.
+    vs.splitAndCalcNormals () ;
+
+    // Generate the new vertices by cloning their originals
+    int newVerts = vs.newVerts () ;
+    if ( vnum + newVerts > MAX_OPT_VERTEX_LIST )
+        return; // Oh well.  No harm done, at least.
+
+    for ( i = 0 ; i < newVerts ; i++ ) {
+        OptVertex *ov = vlist[ vs.origVert ( i ) ] ;
+        vlist[ i + vnum ] = new OptVertex ( ov );
+    }
+    vnum += newVerts;
+
+    // Copy out the new, optimized normal data
+    for(i=0; i<vnum; i++)
+        sgCopyVec3(vlist[i]->normal, vs.norm(i));
+
+    // Zero out the reference counts for the vertices; they will have
+    // changed during the split and will be recalculated below.  This
+    // is an ugly hack -- a more elegant solution would be to do the
+    // bump()/dent() calls inside the ssgVertSplitter code as each vertex
+    // is duplicated.
+    for ( i = 0 ; i < vnum ; i++ )
+        while ( vlist [ i ] -> getCount () )
+            vlist [ i ] -> dent () ;
+
+    // Copy out the possibly changed vertices for the triangles
+    for (i = 0 ; i < tnum ; i++ ) {
+        short* oldtri = tlist + 3 * i ;
+        int* newtri = vs.getTri ( i ) ;
+        for ( j = 0 ; j < 3 ; j++ ) {
+            oldtri [ j ] = newtri [ j ] ;
+            vlist[ newtri [ j ] ] -> bump ();
+        }
+    }
 }
 
 short OptVertexList::find ( sgVec3 v, sgVec2 t, sgVec4 c, int tex_frac )
@@ -669,7 +709,7 @@ void ssgStripify ( ssgEntity *ent )
 				GLenum thisType = ((ssgVtxTable *)k)->getPrimitiveType ();
 				if ((thisType != GL_POINTS) && (thisType != GL_LINES) &&
 						(thisType != GL_LINE_STRIP) && (thisType != GL_LINE_LOOP))
-        {	list . add ( (ssgVtxTable *) k ) ;
+         {      list . add ( (ssgVtxTable *) k ) ;
 					b_ent -> removeKid ( k ) ;
 					k = b_ent -> getKid ( 0 ) ;
 				}
@@ -750,7 +790,7 @@ void ssgStripify ( ssgEntity *ent )
         new_colours  -> add ( list.vlist[new_vlist[m]]->colour   ) ;
       }
 
-      delete new_vlist ;
+      delete [] new_vlist ;
 
       ssgVtxTable *new_vtable = new ssgVtxTable ( GL_TRIANGLE_STRIP,
                     new_coords, new_normals, new_texcoords, new_colours ) ;
@@ -761,8 +801,8 @@ void ssgStripify ( ssgEntity *ent )
     }
   }
 
-  delete  slist ;
-  delete cflist ;
+  delete []  slist ;
+  delete [] cflist ;
 }
 
 
@@ -778,6 +818,9 @@ static void safe_replace_kid ( ssgBranch *parent, ssgEntity *old_kid, ssgEntity
     If parent is null then loop over all parents of old_kid.
   */
 
+  if ( old_kid == new_kid )
+    return ;
+
   if ( parent == NULL )
   {
     int n = old_kid -> getNumParents () ;
@@ -786,26 +829,19 @@ static void safe_replace_kid ( ssgBranch *parent, ssgEntity *old_kid, ssgEntity
     return ;
   }
 
-  assert ( parent -> searchForKid ( old_kid ) >= 0 ) ;
+  // assert ( parent -> searchForKid ( old_kid ) >= 0 ) ;
 
   if ( new_kid == NULL )
   {
-    if ( parent -> isAKindOf ( ssgTypeRangeSelector () ) )
-    {
-      ssgRangeSelector *s = (ssgRangeSelector *) parent ;
-      for ( int i = s -> searchForKid ( old_kid ) ; 
-	        i < s -> getNumKids () ; i++ )
-	s -> setRange ( i, s -> getRange ( i + 1 ) ) ;
-    }
-
-    else if ( parent -> isAKindOf ( ssgTypeSelector () ) )
+    if ( parent -> isAKindOf ( ssgTypeSelector () ) )
     { 
-      /* ssgTimedSelector among others -- jarnspikar */
+      /* cannot remove kids from selectors */
       parent -> replaceKid ( old_kid, new ssgInvisible ) ;
-      return ;
     }
-
-    parent -> removeKid ( old_kid ) ;
+    else
+    {
+      parent -> removeKid ( old_kid ) ;
+    }
   }
   else
   {
@@ -880,9 +916,8 @@ static void flatten ( ssgBranch *parent, ssgEntity *ent, sgMat4 mat )
   */
   if ( ent -> isAKindOf ( ssgTypeCutout () ) ||
        ent -> isAKindOf ( ssgTypeRangeSelector () ) ||
-       ( ent -> isAKindOf ( ssgTypeTransform () ) &&
-	 ( ent -> getUserData () != NULL || 
-	   ! ent -> isA ( ssgTypeTransform () ) )))
+       ( ent -> isA ( ssgTypeTransform () ) &&
+         ent -> getUserData () != NULL ) )
   {
     /* Insert a transform node if needed. */
     if ( mat != NULL ) {
@@ -936,7 +971,8 @@ static void flatten ( ssgBranch *parent, ssgEntity *ent, sgMat4 mat )
     t_ent -> getTransform ( mat2 ) ;
     if ( mat != NULL )
       sgPostMultMat4 ( mat2, mat ) ;
-    mat = mat2 ;
+
+    mat = sgClassifyMat4 ( mat2 ) != 0 ? mat2 : NULL ;
     
     ssgBranch *br = new ssgBranch ;
     /*
