@@ -1,6 +1,6 @@
 /*
      PLIB - A Suite of Portable Game Libraries
-     Copyright (C) 2001  Steve Baker
+     Copyright (C) 1998,2002  Steve Baker
  
      This library is free software; you can redistribute it and/or
      modify it under the terms of the GNU Library General Public
@@ -13,12 +13,12 @@
      Library General Public License for more details.
  
      You should have received a copy of the GNU Library General Public
-     License along with this library; if not, write to the Free
+     License along with this library; if not, write to the Free Software
      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: ssgVtxTable.cxx,v 1.18 2001/06/16 16:27:37 stromberg Exp $
+     $Id: ssgVtxTable.cxx,v 1.34 2004/02/02 01:35:15 sjbaker Exp $
 */
 
 
@@ -26,6 +26,106 @@
 
 #define HL_DELTA 0.04f
 
+int ssgSimpleList::compare(ssgSimpleList *other, int print_result)
+{
+	int result = TRUE;
+  if (total   !=other->total   )
+	{	result = FALSE;
+		if(print_result)
+			printf("ERROR: total   \n");
+	}
+	else
+		if(print_result)
+		  printf("Ok: total   \n");
+  if (size_of !=other->size_of )
+	{	result = FALSE;
+		if(print_result)
+		  printf("ERROR: size_of \n");
+	}
+	else
+		if(print_result)
+		  if(print_result)
+		    printf("Ok: size_of \n");
+	if (memcmp(list, other->list, total*size_of))
+	{	result = FALSE;
+		if(print_result)
+		  printf("ERROR: list \n");
+	}
+	else
+		if(print_result)
+		  printf("Ok: list \n");
+
+	return result;
+}
+
+
+int ssgVtxTable::compare(ssgVtxTable *other, int print_result)
+// returns TRUE, if equal.
+// prints stuff to stdout.
+{
+	int result = TRUE;
+	if(print_result)
+		printf("comparing vertices\n");
+  if(!vertices->compare(other->vertices, print_result))
+		result=FALSE;
+  if(print_result)
+		printf("comparing normals\n");
+  if(!normals->compare(other->normals, print_result))
+		result=FALSE;
+  if(print_result)
+		printf("comparing texcoords\n");
+  if(!texcoords->compare(other->texcoords, print_result))
+		result=FALSE;
+  if(print_result)
+		printf("comparing colours \n");
+  if(!colours ->compare(other->colours, print_result))
+		result=FALSE;
+  // could be in ssgLeaf:
+  if(getState () == other->getState ())
+	{
+		if(print_result)
+		  printf("Ok: state\n");
+	}
+  else
+	  if(print_result)
+		  printf("WARNING: state pointers differ. state contents is not investigated\n");
+  
+  if (getNumVertices  ()!=other->getNumVertices  ())
+	{	result = FALSE;
+		if(print_result)
+		  printf("ERROR: getNumVertices  ()\n");
+	}
+	else
+		if(print_result)
+		  printf("Ok: getNumVertices  ()\n");
+  if (getNumNormals   ()!=other->getNumNormals   ())
+	{	result = FALSE;
+		if(print_result)
+		  printf("ERROR: getNumNormals   ()\n");
+	}
+	else
+		if(print_result)
+		  printf("Ok: getNumNormals   ()\n");
+  if (getNumColours   ()!=other->getNumColours   ())
+	{	result = FALSE;
+		if(print_result)
+		  printf("ERROR: getNumColours   ()\n");
+	}
+	else
+		if(print_result)
+		  printf("Ok: getNumColours   ()\n");
+  if (getNumTexCoords ()!=other->getNumTexCoords ())
+	{	result = FALSE;
+		if(print_result)
+		  printf("ERROR: getNumTexCoords ()\n");
+	}
+	else
+		if(print_result)
+		  printf("Ok: getNumTexCoords ()\n");
+  // end could be in leaf
+	return result;
+}
+
 void ssgVtxTable::copy_from ( ssgVtxTable *src, int clone_flags )
 {
   ssgLeaf::copy_from ( src, clone_flags ) ;
@@ -182,9 +282,9 @@ void ssgVtxTable::getTriangle ( int n, short *v1, short *v2, short *v3 )
       return ;
 
     case GL_QUADS :
-      *v1 = (n/2)*4 + (n&1) + 0 ;
-      *v2 = (n/2)*4 + (n&1) + 1 ;
-      *v3 = (n/2)*4 + (n&1) + 2 ;
+      *v1 = n*2 + 0 ;
+      *v2 = n*2 + 1 ;
+      *v3 = n*2 + 2 - (n&1)*4 ;
       return ;
 
     default : return ;
@@ -286,16 +386,78 @@ void ssgVtxTable::getLine ( int n, short *v1, short *v2 )
   return ;   
 }
 
+
 void ssgVtxTable::transform ( const sgMat4 m )
 {
   int i ;
+  int flags = sgClassifyMat4 ( m ) ;
+
+  if ( flags == 0 )
+    return ;
+
+
+  if ( ( flags & SG_PROJECTION ) )
+    ulSetError ( UL_WARNING, "ssgVtxTable: Projection matrices currently not supported." ) ;
+  /*
+    note: it is possible to handle projections, but for each normal we would
+    have to know the corresponding vertex coordinates. setting:
+        n[3] = dot(v, n) / v[3]
+    and then transforming with the full 4x4 transposed inverse matrix would do it.
+    see the OpenGL spec.
+  */
+
 
   for ( i = 0 ; i < getNumVertices() ; i++ )
     sgXformPnt3 ( vertices->get(i), vertices->get(i), m ) ;
 
+
+  sgMat4 w ;
+
+  if ( ( flags & ( SG_SCALE | SG_MIRROR | SG_NONORTHO ) ) )
+  {
+    if ( ( flags & SG_NONORTHO ) )
+    {
+      // use the transposed adjoint matrix (only the upper 3x3 is needed)
+      sgVectorProductVec3 ( w[0], m[1], m[2] ) ;
+      sgVectorProductVec3 ( w[1], m[2], m[0] ) ;
+      sgVectorProductVec3 ( w[2], m[0], m[1] ) ;
+    }
+    else
+    {
+      SGfloat scale = SG_ONE ;
+
+      if ( ( flags & SG_SCALE ) )
+      {
+	// prescale matrix to avoid renormalisation
+	scale = scale / sgLengthVec3 ( m[0] ) ;
+      }
+
+      if ( ( flags & SG_MIRROR ) )
+      {
+	// negate to keep normals consistent with triangle orientations
+	scale = - scale ;
+      }
+
+      sgScaleVec3 ( w[0], m[0], scale ) ;
+      sgScaleVec3 ( w[1], m[1], scale ) ;
+      sgScaleVec3 ( w[2], m[2], scale ) ;
+    }
+
+    m = w ;
+  }
+
+
   for ( i = 0 ; i < getNumNormals() ; i++ )
     sgXformVec3 ( normals->get(i), normals->get(i), m ) ;
 
+
+  if ( ( flags & SG_NONORTHO ) )
+  {
+    for ( i = 0 ; i < getNumNormals() ; i++ )
+      sgNormaliseVec3 ( normals->get(i) ) ;
+  }
+
+
   recalcBSphere () ;
 }
 
@@ -377,7 +539,11 @@ void ssgVtxTable::draw ()
   if ( ! preDraw () )
     return ;
 
-  if ( hasState () ) getState () -> apply () ;
+  if ( _ssgCurrentContext-> stateOverridden () )
+     _ssgCurrentContext -> overriddenState () -> apply () ;
+  else
+  if ( hasState () )
+    getState () -> apply () ;
 
   stats_num_leaves++ ;
   stats_num_vertices += getNumVertices() ;
@@ -427,32 +593,44 @@ void ssgVtxTable::pick ( int baseName )
 
 void ssgVtxTable::draw_geometry ()
 {
+  int num_vertices  = getNumVertices  () ;
   int num_colours   = getNumColours   () ;
   int num_normals   = getNumNormals   () ;
-  int num_vertices  = getNumVertices  () ;
   int num_texcoords = getNumTexCoords () ;
 
-  sgVec3 *vx = (sgVec3 *) vertices  -> get(0) ;
-  sgVec3 *nm = (sgVec3 *) normals   -> get(0) ;
-  sgVec2 *tx = (sgVec2 *) texcoords -> get(0) ;
-  sgVec4 *cl = (sgVec4 *) colours   -> get(0) ;
-
-  glBegin ( gltype ) ;
-
   if ( num_colours == 0 ) glColor4f   ( 1.0f, 1.0f, 1.0f, 1.0f ) ;
-  if ( num_colours == 1 ) glColor4fv  ( cl [ 0 ] ) ;
-  if ( num_normals == 1 ) glNormal3fv ( nm [ 0 ] ) ;
+  if ( num_colours == 1 ) glColor4fv  ( colours -> get(0) ) ;
+  if ( num_normals == 1 ) glNormal3fv ( normals -> get(0) ) ;
   
-  for ( int i = 0 ; i < num_vertices ; i++ )
-  {
-    if ( num_colours   > 1 ) glColor4fv    ( cl [ i ] ) ;
-    if ( num_normals   > 1 ) glNormal3fv   ( nm [ i ] ) ;
-    if ( num_texcoords > 1 ) glTexCoord2fv ( tx [ i ] ) ;
+  glPushClientAttrib ( GL_CLIENT_VERTEX_ARRAY_BIT ) ;
 
-    glVertex3fv ( vx [ i ] ) ;
+  if ( num_colours > 1 )
+  {
+    glEnableClientState ( GL_COLOR_ARRAY ) ;
+    glColorPointer ( 4, GL_FLOAT, 0, colours->get(0) ) ;
   }
- 
-  glEnd () ;
+
+  if ( num_normals > 1 )
+  {
+    glEnableClientState ( GL_NORMAL_ARRAY ) ;
+    glNormalPointer ( GL_FLOAT, 0, normals->get(0) ) ;
+  }
+
+  if ( num_texcoords > 1 )
+  {
+    glEnableClientState ( GL_TEXTURE_COORD_ARRAY ) ;
+    glTexCoordPointer ( 2, GL_FLOAT, 0, texcoords->get(0) ) ;
+  }
+
+  glEnableClientState ( GL_VERTEX_ARRAY ) ;
+  glVertexPointer ( 3, GL_FLOAT, 0, vertices->get(0) ) ;
+
+  _ssgCurrStatistics . bumpVertexCount ( num_vertices ) ;
+  _ssgCurrStatistics . bumpLeafCount   ( 1 ) ;
+
+  glDrawArrays ( gltype, 0, num_vertices ) ;
+
+  glPopClientAttrib () ;
 }
 
 
@@ -465,7 +643,7 @@ void ssgVtxTable::hot_triangles ( sgVec3 s, sgMat4 m, int /* test_needed */ )
 
   for ( int i = 0 ; i < nt ; i++ )
   {
-    short   v1,  v2,  v3 ;
+    short v1,  v2,  v3 ;
     sgVec3 vv1, vv2, vv3 ;
     sgVec4 plane ;
 
@@ -497,20 +675,20 @@ void ssgVtxTable::hot_triangles ( sgVec3 s, sgMat4 m, int /* test_needed */ )
         continue ;
 
       /* Find the point vertically below the text point
-	as it crosses the plane of the polygon */
+      as it crosses the plane of the polygon */
 
       float z = sgHeightOfPlaneVec2 ( plane, s ) ;
 
       /* No HOT from below the triangle */
 
       if ( z > s[2] )
-	continue ;
+         continue ;
 
       /* Outside the vertical extent of the triangle? */
 
       if ( ( z < vv1[2] && z < vv2[2] && z < vv3[2] ) ||
-	   ( z > vv1[2] && z > vv2[2] && z > vv3[2] ) )
-	continue ;
+         ( z > vv1[2] && z > vv2[2] && z > vv3[2] ) )
+         continue ;
     }
 
     /*
@@ -538,6 +716,102 @@ void ssgVtxTable::hot_triangles ( sgVec3 s, sgMat4 m, int /* test_needed */ )
   }
 }
 
+void ssgVtxTable::los_triangles ( sgVec3 s, sgMat4 m, int /* test_needed */ )
+{
+  int nt = getNumTriangles () ;
+
+  stats_los_triangles += nt ;
+
+  for ( int i = 0 ; i < nt ; i++ )
+  {
+    short v1,  v2,  v3 ;
+    sgVec3 vv1, vv2, vv3 ;
+    sgVec4 plane ;
+
+    SGfloat edge1[3], edge2[3], tvec[3], pvec[3], qvec[3];
+    SGfloat det,inv_det;
+    SGfloat /*t,*/u,v;
+
+    getTriangle ( i, &v1, &v2, &v3 ) ;
+
+    sgXformPnt3 ( vv1, getVertex(v1), m ) ;
+    sgXformPnt3 ( vv2, getVertex(v2), m ) ;
+    sgXformPnt3 ( vv3, getVertex(v3), m ) ;
+    sgVec3 cam;
+    cam[0] = m[0][3];
+    cam[1] = m[1][3];
+    cam[2] = m[2][3];
+    //if ( _ssgIsLosTest )
+    //{
+
+      /* find vectors for two edges sharing vert0 */
+      sgSubVec3(edge1, vv2, vv1);
+      sgSubVec3(edge2, vv3, vv1);
+
+      /* begin calculating determinant - also used to calculate U parameter */
+      sgVectorProductVec3(pvec, s, edge2);
+
+      /* if determinant is near zero, ray lies in plane of triangle */
+      det = sgScalarProductVec3(edge1, pvec);
+
+      if (det > -0.0000001 && det < 0.0000001) continue;
+      inv_det = (float)1.0 / det;
+
+      /* calculate distance from vert0 to ray origin */
+      sgSubVec3(tvec, cam, vv1);
+
+      /* calculate U parameter and test bounds */
+      u = sgScalarProductVec3(tvec, pvec) * inv_det;
+      if (u < 0.0 || u > 1.0)
+         continue;
+
+      /* prepare to test V parameter */
+      sgVectorProductVec3(qvec, tvec, edge1);
+
+      /* calculate V parameter and test bounds */
+      v = sgScalarProductVec3(s, qvec) * inv_det;
+      if (v < 0.0 || u + v > 1.0)
+         continue;
+
+      /* calculate t, ray intersects triangle */
+      //t = sgScalarProductVec3(edge2, qvec) * inv_det;
+    //}
+
+    sgMakePlane ( plane, vv1, vv2, vv3 ) ;
+    _ssgAddHit ( this, i, m, plane ) ;
+  }
+}
+
+
+ssgVtxArray *ssgVtxTable::getAs_ssgVtxArray()
+{ 
+
+	ssgIndexArray    *indices = new ssgIndexArray ();
+	int i, no = 0;
+	switch ( getPrimitiveType () )
+  {
+    case GL_POLYGON :
+    case GL_TRIANGLE_FAN :
+    case GL_TRIANGLE_STRIP :
+      no = getNumTriangles() + 2 ;
+			break;
+
+    case GL_TRIANGLES :
+      no = getNumTriangles() * 3 ;
+	}
+	for(i=0;i<no;i++)
+		indices ->add(i);
+
+  
+	ssgState * state = getState();
+	char * name = getName();
+	ssgVtxArray * s = new ssgVtxArray ( getGLtype(),
+							vertices, normals, texcoords, colours, indices);
+	s->setName(name);
+	s->setState(state);
+	return s;
+}
+
 
 void ssgVtxTable::isect_triangles ( sgSphere *s, sgMat4 m, int test_needed )
 {
@@ -547,7 +821,7 @@ void ssgVtxTable::isect_triangles ( sgSphere *s, sgMat4 m, int test_needed )
 
   for ( int i = 0 ; i < nt ; i++ )
   {
-    short   v1,  v2,  v3 ;
+    short v1,  v2,  v3 ;
     sgVec3 vv1, vv2, vv3 ;
     sgVec4 plane ;
 
@@ -663,10 +937,25 @@ void ssgVtxTable::print ( FILE *fd, char *indent, int how_much )
 	
   ssgLeaf  ::print ( fd, indent, how_much ) ;
 		
-  vertices  -> print ( fd, in, how_much ) ;
-  normals   -> print ( fd, in, how_much ) ;
-  texcoords -> print ( fd, in, how_much ) ;
-  colours   -> print ( fd, in, how_much ) ;
+  if ( vertices == NULL )
+    fprintf ( fd, "%s  No Vertices!\n", indent ) ;
+  else
+    vertices  -> print ( fd, in, how_much ) ;
+
+  if ( normals == NULL )
+    fprintf ( fd, "%s  No Normals!\n", indent ) ;
+  else
+    normals   -> print ( fd, in, how_much ) ;
+
+  if ( texcoords == NULL )
+    fprintf ( fd, "%s  No Texcoords!\n", indent ) ;
+  else
+    texcoords -> print ( fd, in, how_much ) ;
+
+  if ( colours == NULL )
+    fprintf ( fd, "%s  No Colours!\n", indent ) ;
+  else
+    colours   -> print ( fd, in, how_much ) ;
 }
 
 
